// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: TableStatistics
// Author: Hassan Siddiqui
//
// Description: Compute statistics for a given table
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: TableStatistics.cc,v 1.5 2001/12/18 11:26:54 gvacanti Exp $


/* General */
#include <iostream>
#include <string>
#include <map>
#include <vector>

/* SAS-specific */
#include "Dal.h"
#include "errstr.h"
#include "Statistic.h"
#include "Tracer.h"

/* Local */
#include "TableStatistics.h"

/* Constructors */

TableStatistics:: TableStatistics(){};
 
TableStatistics:: TableStatistics(const Table& table, 
				  const vector<string>& columnNames, 
				  const vector<RowRange>& rowranges) 
{
  PROC;
  
  generateStatistics(table, columnNames, rowranges);

  ENDPROC;
}
	
TableStatistics:: TableStatistics(const Table& table, 
				  const vector<string>& columnNames) 
{
  PROC;
  
  RowRange rowrange;
  rowrange.startRow = 0;
  rowrange.stopRow  = table.rows()-1;
  
  vector<RowRange> rowranges;
  
  rowranges.push_back(rowrange);
  
  generateStatistics(table, columnNames, rowranges);

  ENDPROC;

}
	
/* Output stream method */
void TableStatistics:: write(ostream& stream) const
{
  PROC;
  
  for (TableStatistics::const_iterator i = begin(); i != end(); ++i)
    {
//        stream << (*i).first << endl 
//  	     << (*i).second <<endl ;
        stream << (*i).second << ends;
    }

  ENDPROC;
}

void TableStatistics::generateStatistics(const Table& table, 
					 const vector<string>& columnNames, 
					 const vector<RowRange>& rowranges) 
{
  PROC;

  /* Create map of statistic objects */
  for (vector<string>::const_iterator name = columnNames.begin(); 
       name != columnNames.end(); 
       ++name)
    {
      /* Get column data */
      if (!table.hasColumn(*name))
	{
	  errstr << "Column " << *name 
		 << " does not exist. Will not generate an output for this." 
		 << Sas::warning("invalidColumnName");
	  continue;
	}
	  

      /* Compute statistics on column  */
      Statistic statistic(table, *name, rowranges);

      /* Add statistics object to map */
      (*this)[*name] = statistic;
	  
    }

  ENDPROC;
}



/* I/O Stream functions */

ostream & operator << (ostream& stream, 
  const TableStatistics& tablestatistics)
{
  PROC;
  tablestatistics.write(stream);
  RETURN stream;
}


